
#navigator-throbber
  {
    background-image      : url("chrome://communicator/skin/navbar-endbox.gif");
    background-repeat     : no-repeat;
    list-style-image      : url("chrome://global/skin/animthrob_single.gif");
    width                 : 68px;
    max-width             : 68px;
    padding               : 11px 3px 1px 23px;
    margin                : 0px;
    cursor                : pointer;
  }  

#navigator-throbber[busy="true"]
  {
    list-style-image      : url("chrome://global/skin/animthrob.gif");
  }
  
#navigator-throbber:hover:active
  {
    padding               : 12px 2px 0px 24px;
    margin                : 0px;
  }

