
titledbox
  {
    border                : 2px groove threedface;
    padding               : 7px;
    margin                : 5px;
  }

title
  {
    margin-left           : 6px;
    padding-right         : 40px;
  }

/* This just SUCKS, but seems to be necessitated by titledbox design... */
title > *
  {
    margin-left           : 5px;
    margin-right          : 5px;
  }

:titledbox-content 
  {
    display               : block;
    padding               : inherit;
  }

    
/** box formatting **/
  box[autostretch="never"]
    {
      vertical-align      : middle;
    }
    
  box[autostretch="never"][valign="top"]
    {
      vertical-align      : top;
    }
    
  box[autostretch="never"][valign="bottom"]
    {
      vertical-align      : bottom;
    }
    
  box[autostretch="never"][halign="center"] 
    {
      text-align          : center;
    }
    
  box[autostretch="never"][halign="right"]
    {
      text-align          : right;
    }

  row
    {
      vertical-align      : middle;
    }
  